import { PrismaClient } from '@prisma/client';
import { AuthenticationError, UserInputError } from 'apollo-server-express';
// In a real application, you would use bcrypt for password hashing
// import bcrypt from 'bcryptjs';
// import jwt from 'jsonwebtoken';

const prisma = new PrismaClient();

export const userResolvers = {
  Query: {
    me: async (_: any, __: any, context: any) => {
      // In a real application, you would get the user from the context
      // which would be set by an authentication middleware
      if (!context.userId) {
        return null;
      }
      return prisma.user.findUnique({
        where: { id: context.userId },
      });
    },
    users: async () => {
      return prisma.user.findMany();
    },
    user: async (_: any, { id }: { id: string }) => {
      return prisma.user.findUnique({
        where: { id },
      });
    },
  },
  Mutation: {
    register: async (
      _: any,
      {
        email,
        username,
        password,
      }: { email: string; username: string; password: string }
    ) => {
      // Check if user already exists
      const existingUser = await prisma.user.findFirst({
        where: {
          OR: [{ email }, { username }],
        },
      });

      if (existingUser) {
        throw new UserInputError('User already exists');
      }

      // In a real application, you would hash the password
      // const hashedPassword = await bcrypt.hash(password, 10);

      const user = await prisma.user.create({
        data: {
          email,
          username,
          password, // In a real app: hashedPassword
        },
      });

      // In a real application, you would generate a JWT token
      // const token = jwt.sign({ userId: user.id }, process.env.JWT_SECRET);

      return {
        token: 'dummy-token', // In a real app: token
        user,
      };
    },
    login: async (
      _: any,
      { email, password }: { email: string; password: string }
    ) => {
      const user = await prisma.user.findUnique({
        where: { email },
      });

      if (!user) {
        throw new AuthenticationError('Invalid credentials');
      }

      // In a real application, you would verify the password
      // const valid = await bcrypt.compare(password, user.password);
      const valid = password === user.password; // This is just for demo purposes

      if (!valid) {
        throw new AuthenticationError('Invalid credentials');
      }

      // In a real application, you would generate a JWT token
      // const token = jwt.sign({ userId: user.id }, process.env.JWT_SECRET);

      return {
        token: 'dummy-token', // In a real app: token
        user,
      };
    },
  },
  User: {
    messages: async (parent: any) => {
      return prisma.message.findMany({
        where: { userId: parent.id },
      });
    },
    rooms: async (parent: any) => {
      return prisma.room.findMany({
        where: {
          members: {
            some: {
              id: parent.id,
            },
          },
        },
      });
    },
    ownedRooms: async (parent: any) => {
      return prisma.room.findMany({
        where: { ownerId: parent.id },
      });
    },
  },
};
